@@ -1,8 +1,11 @@
+import glob
 import os
+import shutil
+import sys
 import warnings
 from abc import abstractmethod
+from collections import Counter
 from typing import Any, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
-from uuid import uuid4
 
 import numpy as np
 import pandas as pd
@@ -14,18 +17,18 @@ from dataquality.clients.objectstore import ObjectStore
 from dataquality.core._config import config
 from dataquality.exceptions import GalileoException, GalileoWarning
 from dataquality.loggers.base_logger import BaseGalileoLogger, BaseLoggerAttributes
-from dataquality.schemas.dataframe import BaseLoggerInOutFrames
+from dataquality.schemas.dataframe import BaseLoggerInOutFrames, DFVar
 from dataquality.schemas.ner import TaggingSchema
 from dataquality.schemas.split import Split
 from dataquality.utils import tqdm
 from dataquality.utils.cloud import is_galileo_cloud
 from dataquality.utils.hdf5_store import HDF5_STORE
+from dataquality.utils.helpers import galileo_verbose_logging
 from dataquality.utils.thread_pool import ThreadPoolManager
 from dataquality.utils.vaex import (
     _join_in_out_frames,
     concat_hdf5_files,
     filter_df,
-    validate_ids_for_df,
     validate_unique_ids,
 )
 
@@ -39,7 +42,7 @@ ITER_CHUNK_SIZE = 100_000
 class BaseGalileoDataLogger(BaseGalileoLogger):
     MAX_META_COLS = 25  # Limit the number of metadata attrs a user can log
     MAX_STR_LEN = 100  # Max characters in a string metadata attribute
-    INPUT_DATA_NAME = "input_data.arrow"
+    INPUT_DATA_BASE = "input_data"
     MAX_DATA_SIZE_CLOUD = 300_000
 
     DATA_FOLDER_EXTENSION = {data_folder: "hdf5" for data_folder in DATA_FOLDERS}
@@ -49,6 +52,17 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         self.meta: Dict = meta or {}
         self.log_export_progress = True
 
+    def input_data_path(self) -> str:
+        return f"{self.write_output_dir()}/{BaseGalileoDataLogger.INPUT_DATA_BASE}"
+
+    def input_data_file(self, input_num: int = None, split: str = None) -> str:
+        if not split:
+            assert self.split
+            split = str(self.split)
+        if input_num is None:
+            input_num = self.logger_config.input_data_logged[split]
+        return f"{self.input_data_path()}/{split}/data_{input_num}.arrow"
+
     @abstractmethod
     def log_data_sample(self, *, text: str, id: int, **kwargs: Any) -> None:
         """Log a single input sample. See child for details"""
@@ -75,83 +89,74 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
 
         Provide the dataset and the keys to index into it. See child for details"""
 
+    def validate_ids_for_split(self, ids: List[int]) -> None:
+        split = self.split_name()
+        exc = (
+            "If you've re-run a block of code or notebook cell that logs model "
+            "outputs, that could be the cause. Try reinitializing with `dq.init` "
+            "to clear your local environment, and then logging your data again. Call "
+            "`dq.enable_galileo_verbose()` to see the duplicate IDs"
+        )
+        id_set = set(ids)
+        if len(id_set) != len(ids):
+            exc = "It seems you do not have unique ids in this logged data. " + exc
+            dups = {k: v for k, v in Counter(ids).items() if v > 1}
+            if galileo_verbose_logging():
+                exc += f"split:{split}, dup ids and counts: {dups}"
+            raise GalileoException(exc)
+        # This means some logged ids were already logged!
+        if len(id_set - self.logger_config.logged_input_ids[split]) != len(ids):
+            extra = self.logger_config.logged_input_ids[split].intersection(id_set)
+            exc = "Some ids in this dataset were already logged for this split. " + exc
+            if galileo_verbose_logging():
+                exc += f"split:{split}, overlapping ids: {extra}"
+            raise GalileoException(exc)
+        self.logger_config.logged_input_ids[split].update(ids)
+
     def log(self) -> None:
         """Writes input data to disk in .galileo/logs
 
         If input data already exist, append new data to existing input file
         """
         self.validate()
-        write_input_dir = (
-            f"{BaseGalileoLogger.LOG_FILE_DIR}/{config.current_project_id}/"
-            f"{config.current_run_id}"
-        )
-        if not os.path.exists(write_input_dir):
-            os.makedirs(write_input_dir)
+        write_input_dir = self.write_output_dir()
+        os.makedirs(write_input_dir, exist_ok=True)
+        os.makedirs(f"{self.input_data_path()}/{self.split}", exist_ok=True)
 
         df = self._get_input_df()
-        file_path = f"{write_input_dir}/{BaseGalileoDataLogger.INPUT_DATA_NAME}"
-
-        if os.path.isfile(file_path):
-            self.append_input_data(df, write_input_dir, file_path)
-        else:
-            self.validate_data_size(df)
-            if self.log_export_progress:
-                with vaex.progress.tree("vaex", title="Exporting input data"):
-                    df.export(file_path)
-            else:
-                df.export(file_path)
-        df.close()
-
-    def append_input_data(
-        self, df: DataFrame, write_input_dir: str, file_path: str
-    ) -> None:
-        # Create a temporary file for existing input data
-        tmp_name = f"{write_input_dir}/{str(uuid4()).replace('-', '')[:12]}.arrow"
-        os.rename(file_path, tmp_name)
-
-        # Merge existing data with new data
-        existing_df = vaex.open(tmp_name)
-        merged_df = vaex.concat([existing_df, df])
-        self.validate_data_size(merged_df)
-
-        try:  # Validate there are no duplicated IDs
-            validate_ids_for_df(merged_df)
-        except GalileoException as e:  # Cleanup and raise on error
-            merged_df.close()
-            os.rename(tmp_name, file_path)  # Revert name, we aren't logging
-            raise e
+        self.validate_data_size(df)
+        self.validate_ids_for_split(df["id"].tolist())
 
+        file_path = self.input_data_file()
         if self.log_export_progress:
-            with vaex.progress.tree("vaex", title="Appending input data"):
-                merged_df.export(file_path)
+            with vaex.progress.tree("vaex", title=f"Logging {len(df)} samples"):
+                df.export(file_path)
         else:
-            merged_df.export(file_path)
+            df.export(file_path)
 
-        # Cleanup temporary file after appending input data
-        os.remove(tmp_name)
+        df.close()
+        self.logger_config.input_data_logged[str(self.split)] += 1
 
-    @classmethod
-    def upload(cls, last_epoch: Optional[int] = None) -> None:
+    def upload(self, last_epoch: Optional[int] = None) -> None:
         """
         Iterates through all of each splits children folders [data/emb/prob] for each
         inference name / epoch, concatenates all of the files with vaex, and uploads
         them to a single file in minio
         """
         ThreadPoolManager.wait_for_threads()
-        cls.check_for_logging_failures()
+        self.check_for_logging_failures()
         print("☁️ Uploading Data")
         object_store = ObjectStore()
         proj_run = f"{config.current_project_id}/{config.current_run_id}"
-        location = f"{cls.LOG_FILE_DIR}/{proj_run}"
+        location = f"{self.LOG_FILE_DIR}/{proj_run}"
 
-        in_frame = vaex.open(
-            f"{location}/{BaseGalileoDataLogger.INPUT_DATA_NAME}"
-        ).copy()
         for split in Split.get_valid_attributes():
             split_loc = f"{location}/{split}"
-            if not os.path.exists(split_loc):
+            input_logged = os.path.exists(f"{self.input_data_path()}/{split}")
+            output_logged = os.path.exists(split_loc)
+            if not output_logged:
                 continue
-            if not len(in_frame[in_frame["split"] == split]):
+            if not input_logged:
                 warnings.warn(
                     f"There was output data logged for split {split} but no input data "
                     "logged. Skipping upload for this split as there are no samples "
@@ -159,9 +164,13 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
                     GalileoWarning,
                 )
                 continue
-
-            in_frame_split = filter_df(in_frame, "split", split)
-            cls.upload_split(object_store, in_frame_split, split, split_loc, last_epoch)
+            in_frame_path = f"{self.input_data_path()}/{split}"
+            in_frame_split = vaex.open(f"{in_frame_path}/*.arrow")
+            self.upload_split(
+                object_store, in_frame_split, split, split_loc, last_epoch
+            )
+            in_frame_split.close()
+            shutil.rmtree(in_frame_path)
 
     @classmethod
     def upload_split(
@@ -184,25 +193,30 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         epochs_or_infs = epochs_or_infs[:last_epoch]
 
         # For each inference name or epoch of the given split
-        for split_run in tqdm(epochs_or_infs, total=len(epochs_or_infs), desc=split):
-            in_frame_slice = in_frame.copy()
-            prob_only = cls.prob_only(epochs_or_infs, split, split_run)
+        for epoch_or_inf in tqdm(
+            epochs_or_infs,
+            total=len(epochs_or_infs),
+            desc=split,
+            file=sys.stdout,
+        ):
+            input_batch = in_frame.copy()
+            prob_only = cls.prob_only(epochs_or_infs, split, epoch_or_inf)
             if split == Split.inference:
-                if not len(in_frame[in_frame["inference_name"] == split_run]):
+                input_batch = filter_df(input_batch, "inference_name", epoch_or_inf)
+                if not len(input_batch):
                     warnings.warn(
-                        f"There was output data logged for inference_name {split_run} "
-                        "but no input data logged. Skipping upload for this inference "
-                        "run as there are no samples to join to.",
+                        "There was output data logged for inference_name "
+                        f"{epoch_or_inf} but no input data logged. Skipping upload for "
+                        "this inference run as there are no samples to join to.",
                         GalileoWarning,
                     )
                     continue
-                in_frame_slice = filter_df(in_frame_slice, "inference_name", split_run)
 
-            dir_name = f"{split_loc}/{split_run}"
+            dir_name = f"{split_loc}/{epoch_or_inf}"
             in_out_frames = cls.create_in_out_frames(
-                in_frame_slice, dir_name, prob_only, split, split_run
+                input_batch, dir_name, prob_only, split, epoch_or_inf
             )
-            cls.upload_in_out_frames(object_store, in_out_frames, split, split_run)
+            cls.upload_in_out_frames(object_store, in_out_frames, split, epoch_or_inf)
 
     @classmethod
     def create_in_out_frames(
@@ -211,8 +225,22 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         dir_name: str,
         prob_only: bool,
         split: str,
-        split_run: Union[str, int],
+        epoch_or_inf: Union[str, int],
     ) -> BaseLoggerInOutFrames:
+        """Formats the input data and model output data
+
+        In this step, we concatenate the many hdf5 files created during model training
+        and logging. We log those in threaded processes, and here we combine them
+        into a single hdf5 file that vaex can read into a dataframe
+
+        :param in_frame: the input dataframe
+        :param dir_name: The directory of all of the output hdf5 files
+        :param prob_only: If we are only uploading probability data. We only upload
+            probability data for all epochs expect the last one (we dont use cross-epoch
+            embeddings currently, so we dont log them)
+        :param split: The split we are logging for
+        :param epoch_or_inf: The epoch or inference name we are logging for
+        """
         str_cols = concat_hdf5_files(dir_name, prob_only)
         out_frame = vaex.open(f"{dir_name}/{HDF5_STORE}")
 
@@ -231,7 +259,7 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
                 split, length=len(out_frame), dtype="str"
             )
             out_frame[epoch_or_inf_name] = vaex.vconstant(
-                split_run, length=len(out_frame), dtype=dtype
+                epoch_or_inf, length=len(out_frame), dtype=dtype
             )
 
         return cls.process_in_out_frames(
@@ -248,9 +276,14 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
     ) -> BaseLoggerInOutFrames:
         """Processes input and output dataframes from logging
 
-        Validates uniqueness of IDs in the dataframes
+        Validates uniqueness of IDs in the output dataframe
         Joins inputs and outputs
         Splits the dataframes into prob, emb, and data for uploading to minio
+
+        :param in_frame: The input dataframe
+        :param out_frame: The model output dataframe
+        :param prob_only: If we are only uploading probabilities, or everything
+        :param epoch_or_inf_name: The epoch or inference name we are uploading for
         """
         validate_unique_ids(out_frame, epoch_or_inf_name)
         in_out = _join_in_out_frames(in_frame, out_frame)
@@ -270,7 +303,7 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         object_store: ObjectStore,
         in_out_frames: BaseLoggerInOutFrames,
         split: str,
-        split_run: Union[str, int],
+        epoch_or_inf: Union[str, int],
     ) -> None:
         proj_run = f"{config.current_project_id}/{config.current_run_id}"
 
@@ -278,19 +311,23 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         emb = in_out_frames.emb
         data_df = in_out_frames.data
 
-        epoch_inf = prob.variables.pop("progress_name", "")
+        epoch_inf = prob.variables.pop(DFVar.progress_name, "")
 
         name = "inf_name" if split == Split.inference else "epoch"
         desc = f"{split} ({name}={epoch_inf})"
         for data_folder, df_obj in tqdm(
-            zip(DATA_FOLDERS, [emb, prob, data_df]), total=3, desc=desc, leave=False
+            zip(DATA_FOLDERS, [emb, prob, data_df]),
+            total=3,
+            desc=desc,
+            leave=False,
+            file=sys.stdout,
         ):
-            if df_obj.variables.get("skip_upload"):
+            if df_obj.variables.get(DFVar.skip_upload):
                 continue
 
             ext = cls.DATA_FOLDER_EXTENSION[data_folder]
             minio_file = (
-                f"{proj_run}/{split}/{split_run}/{data_folder}/{data_folder}.{ext}"
+                f"{proj_run}/{split}/{epoch_or_inf}/{data_folder}/{data_folder}.{ext}"
             )
             object_store.create_project_run_object_from_df(
                 df=df_obj, object_name=minio_file
@@ -298,13 +335,13 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
 
     @classmethod
     def prob_only(
-        cls, epochs: List[str], split: str, split_run: Union[int, str]
+        cls, epochs: List[str], split: str, epoch_or_inf_name: Union[int, str]
     ) -> bool:
         if split == Split.inference:
             return False
 
-        # If split is not inference, split_run must be epoch
-        epoch = int(split_run)
+        # If split is not inference, epoch_or_inf must be epoch
+        epoch = int(epoch_or_inf_name)
         # For all epochs that aren't the last 2 (early stopping), we only
         # want to upload the probabilities (for DEP calculation).
         max_epoch_for_split = max([int(i) for i in epochs])
@@ -412,12 +449,15 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         """Sets the tagging schema, if applicable. Must be implemented by child"""
         raise GalileoException(f"Cannot set tagging schema for {cls.__logger_name__}")
 
-    @classmethod
-    def validate_data_size(cls, df: DataFrame) -> None:
+    def validate_data_size(self, df: DataFrame) -> None:
         if not is_galileo_cloud():
             return
+        samples_logged = len(df)
+        path_to_logged_data = f"{self.input_data_path()}/*/*arrow"
+        if glob.glob(path_to_logged_data):
+            samples_logged += len(vaex.open(f"{self.input_data_path()}/*/*arrow"))
         nrows = BaseGalileoDataLogger.MAX_DATA_SIZE_CLOUD
-        if len(df) > nrows:
+        if samples_logged > nrows:
             warnings.warn(
                 f"⚠️ Hey there! You've logged over {nrows} rows in your input data. "
                 f"Galileo Cloud only supports up to {nrows} rows. "
