@@ -96,7 +96,7 @@ def test_log_input_examples(set_test_config, cleanup_after_use):
         ]
     )
 
-    logged_data = vaex.open(f"{LOCATION}/input_data.arrow")
+    logged_data = vaex.open(f"{LOCATION}/input_data/training/data_0.arrow")
 
     assert logged_data["id"].tolist() == [0, 1, 2, 3, 4]
     assert logged_data["split"].tolist() == ["training"] * len(training_examples)
@@ -134,7 +134,7 @@ def test_watch(set_test_config, cleanup_after_use):
     nlp.initialize(lambda: training_examples)
     watch(nlp)
 
-    assert text_ner_logger_config.user_data["nlp"] == nlp
+    assert text_ner_logger_config.helper_data["nlp"] == nlp
     assert dataquality.get_data_logger().logger_config.labels == NER_CLASS_LABELS
     assert dataquality.get_data_logger().logger_config.tagging_schema == "BILOU"
 
@@ -156,9 +156,14 @@ def test_unwatch(set_test_config):
 
     # This should be possible here
     pickle.dumps(nlp)
+    # and this
+    pickle.loads(pickle.dumps(nlp.get_pipe("ner")))
+    # and this
+    pickle.loads(pickle.dumps(nlp))
 
-    watch(nlp)
-    unwatch(nlp)
+    for _ in range(3):  # This should be possible multiple times
+        watch(nlp)
+        unwatch(nlp)
 
     unwatched_ner = nlp.get_pipe("ner")
     assert isinstance(unwatched_ner, EntityRecognizer)
